import unittest

class TestUtilsMethods(unittest.TestCase):

    def test_add_secure_hash(self):
        from nps_sdk.utils import add_secure_hash
        params = {'psp_QueryCriteriaId': '100409',
                  'psp_QueryCriteria': 'T',
                  'psp_Version': '2.2',
                  'psp_MerchantId': 'psp_test',
                  'psp_PosDateTime': '2016-12-01 12:00:00'}

        secret_key = "IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6"

        resp = add_secure_hash(params, secret_key)
        self.assertEqual('d2f455e71232d7842c739196f6fa25f7', resp.get('psp_SecureHash'))
        self.assertEqual('psp_test', resp.get('psp_MerchantId'))
        self.assertEqual('100409', resp.get('psp_QueryCriteriaId'))


    def test_create_secure_hash(self):
        from nps_sdk.utils import _create_secure_hash
        secret_key = "IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6"

        params = {'psp_QueryCriteriaId': '100409',
                  'psp_QueryCriteria': 'T',
                  'psp_Version': '2.2',
                  'psp_MerchantId': 'psp_test',
                  'psp_PosDateTime': '2016-12-01 12:00:00'}

        resp = _create_secure_hash(params, secret_key)
        self.assertEqual('d2f455e71232d7842c739196f6fa25f7', resp)

    def test_add_extra_info(self):
        from nps_sdk.utils import add_extra_info
        params = {'psp_QueryCriteriaId': '100409',
                  'psp_QueryCriteria': 'T',
                  'psp_Version': '2.2',
                  'psp_MerchantId': 'psp_test',
                  'psp_PosDateTime': '2016-12-01 12:00:00'}

        resp = add_extra_info('PayOnline_2p',params=params)
        self.assertTrue('psp_MerchantAdditionalDetails' in resp.keys())
        self.assertTrue('SdkInfo' in resp.get('psp_MerchantAdditionalDetails').keys())
        self.assertEqual('Python 1.0', resp.get('psp_MerchantAdditionalDetails').get('SdkInfo'))

    def _test_validate_size(self):
        from nps_sdk.utils import _validate_size
        from nps_sdk.conf.sanititize_struc import key_config
        value = 'Python 1.0'
        key = 'psp_MerchantAdditionalDetails.SdkInfo'
        resp = _validate_size(value=value, k=key)
        self.assertGreaterEqual(key_config.get(key+".max_length"), len(resp))

    def test_mask_credit_card(self):
        from nps_sdk.utils import _mask_c_number
        card_number = """<psp_CardNumber xsi:type="ns2:string">4507990000000011</psp_CardNumber>"""
        resp = _mask_c_number(card_number)
        self.assertEqual("""<psp_CardNumber xsi:type="ns2:string">450799******0011</psp_CardNumber>""", resp)

    def test_mask_cvc(self):
        from nps_sdk.utils import _mask_cvc
        cvc = """<psp_CardSecurityCode xsi:type="ns2:string">123</psp_CardSecurityCode>"""
        resp = _mask_cvc(cvc)
        self.assertEqual("""<psp_CardSecurityCode xsi:type="ns2:string">***</psp_CardSecurityCode>""", resp)

    def test_mask_exp_date(self):
        from nps_sdk.utils import _mask_exp_date
        card_exp_date = """<psp_CardExpDate xsi:type="ns2:string">1712</psp_CardExpDate>"""
        resp = _mask_exp_date(card_exp_date)
        self.assertEqual("""<psp_CardExpDate xsi:type="ns2:string">****</psp_CardExpDate>""", resp)


    def test_mask_data(self):
        from nps_sdk.utils import _mask_data
        datos = """
        <psp_CardExpDate xsi:type="ns2:string">1712</psp_CardExpDate>
        <psp_CardSecurityCode xsi:type="ns2:string">123</psp_CardSecurityCode>
        <psp_CardNumber xsi:type="ns2:string">4507990000000011</psp_CardNumber>
        """
        espected = """
        <psp_CardExpDate xsi:type="ns2:string">****</psp_CardExpDate>
        <psp_CardSecurityCode xsi:type="ns2:string">***</psp_CardSecurityCode>
        <psp_CardNumber xsi:type="ns2:string">450799******0011</psp_CardNumber>
        """
        resp = _mask_data(datos)
        self.assertEqual(espected, resp)

    def test_find_c_numbers(self):
        from nps_sdk.utils import _find_c_numbers
        datos = """
                <psp_CardExpDate xsi:type="ns2:string">1712</psp_CardExpDate>
                <psp_CardSecurityCode xsi:type="ns2:string">123</psp_CardSecurityCode>
                <psp_CardNumber xsi:type="ns2:string">4507990000000011</psp_CardNumber>
                """

        resp = _find_c_numbers(datos)
        self.assertEqual(['4507990000000011</psp_CardNumber>'], resp)

    def test_find_exp_date(self):
        from nps_sdk.utils import _find_exp_date
        datos = """
                <psp_CardExpDate xsi:type="ns2:string">1712</psp_CardExpDate>
                <psp_CardSecurityCode xsi:type="ns2:string">123</psp_CardSecurityCode>
                <psp_CardNumber xsi:type="ns2:string">4507990000000011</psp_CardNumber>
                """
        resp = _find_exp_date(datos)
        self.assertEqual(['1712</psp_CardExpDate>'], resp)

    def test_find_cvc(self):
        from nps_sdk.utils import _find_cvc
        datos = """
                <psp_CardExpDate xsi:type="ns2:string">1712</psp_CardExpDate>
                <psp_CardSecurityCode xsi:type="ns2:string">123</psp_CardSecurityCode>
                <psp_CardNumber xsi:type="ns2:string">4507990000000011</psp_CardNumber>
                """
        resp = _find_cvc(datos)
        self.assertEqual(['123</psp_CardSecurityCode>'], resp)


class TestServices(unittest.TestCase):

    def test_get_not_add_services(self):
        from nps_sdk.services import get_merch_det_not_add_services
        espected = ['QueryTxs', 'Refund', 'SimpleQueryTx', 'Capture', 'ChangeSecretKey', 'NotifyFraudScreeningReview', 'GetIINDetails', 'QueryCardNumber', 'CreatePaymentMethod', 'CreatePaymentMethodFromPayment', 'RetrievePaymentMethod', 'UpdatePaymentMethod', 'DeletePaymentMethod', 'CreateCustomer', 'RetrieveCustomer', 'UpdateCustomer', 'DeleteCustomer', 'RecachePaymentMethodToken', 'CreatePaymentMethodToken', 'RetrievePaymentMethodToken', 'CreateClientSession', 'GetInstallmentsOptions', 'QueryCardDetails']
        resp = get_merch_det_not_add_services()
        self.assertEqual(espected, resp)

class TestIssues(unittest.TestCase):

    def test_secure_hash_w_list_in_root_node(self):
        from nps_sdk.utils import _create_secure_hash

        key = "IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6"

        params = {
            "psp_Version": '2.2',
            "psp_MerchantId": 'psp_test',
            "psp_TxSource": 'WEB',
            "psp_MerchOrderId": 'ORDER999qdw',
            "psp_ReturnURL": 'http://localhost/',
            "psp_FrmLanguage": 'es_AR',
            "psp_Amount": 15050,
            "psp_Currency": '032',
            "psp_Country": 'ARG',
            "psp_Product": 14,
            "psp_PosDateTime": '2016-12-01 12:00:00',
            "psp_Transactions": [{
                "psp_MerchantId": 'psp_test',
                "psp_MerchTxRef": 'ORDER36qx6-3',
                "psp_Product": 14,
                "psp_Amount": 10000,
                "psp_NumPayments": 1
            },{
                "psp_MerchantId": 'psp_test',
                "psp_MerchTxRef": 'ORDER1qd66-3',
                "psp_Product": 14,
                "psp_Amount": 5050,
                "psp_NumPayments": 1
            }]
        }

        resp = _create_secure_hash(params=params, secret_key=key)
        self.assertEqual("a4defbaaaf41d581c8a35014820404df", resp)


    def test_issue_3930_error_500_on_p2p_with_updated_costumer_on_dev(self):
        from nps_sdk.constants import DEVELOPMENT_ENV, SANDBOX_ENV
        import nps_sdk
        import logging
        import uuid
        merch_ref = uuid.uuid4()

        nps_sdk.Configuration.configure(environment=SANDBOX_ENV,
                                        secret_key="IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6",
                                        log_level=logging.INFO, debug=True, cert_verify_peer=False)

        sdk = nps_sdk.Nps()

        merchant_id = 'psp_test'

        params = {'psp_AccountCreatedAt': '2010-10-23',
                  'psp_EmailAddress': 'fedevalle@inclufin.com',
                  'psp_MerchantId': merchant_id,
                  'psp_Person': {'FirstName': 'Federico',
                                 'LastName': 'del Valle',
                                 'PhoneNumber1': '+5491164838755'},
                  'psp_PosDateTime': '2017-02-09 18:42:43',
                  'psp_Version': '2.2'}

        create_customer_response = sdk.create_customer(params)


        params = {
            'psp_AccountCreatedAt': '2010-10-23',
            'psp_Address': {
                'AdditionalInfo': '2 A',
                'City': 'Miami',
                'Country': 'USA',
                'HouseNumber': '1245',
                'StateProvince': 'Florida',
                'Street': 'Av. Collins',
                'ZipCode': '33140'
            },
            'psp_AlternativeEmailAddress': 'jdoe@example.com',
            'psp_CustomerId': create_customer_response.psp_CustomerId,
            'psp_EmailAddress': 'jhon.doe@example.com',
            'psp_MerchantId': merchant_id,
            'psp_PaymentMethod': {
                'CardInputDetails': {
                    'ExpirationDate': '1909',
                    'HolderName': 'VISA',
                    'Number': '4242424242424242',
                    'SecurityCode': '9822'
                },
                'Product': '14'
            },
            'psp_Person': {
                'DateOfBirth': '1979-01-12',
                'FirstName': 'John',
                'Gender': 'M',
                'IDNumber': '54111111',
                'IDType': '200',
                'LastName': 'Doe',
                'MiddleName': 'Michael',
                'Nationality': 'ARG',
                'PhoneNumber1': '+1 011 11111111',
                'PhoneNumber2': '+1 011 22222222'
            },
            'psp_PosDateTime': '2008-01-12 13:05:00',
            'psp_Version': '2.2'
        }

        update_customer_response = sdk.update_customer(params)

        params = {
            'psp_MerchOrderId': merch_ref,
            'psp_VaultReference': {
                'CustomerId': update_customer_response.psp_CustomerId
            },
            'psp_MerchTxRef': merch_ref,
            'psp_PosDateTime': '2017-03-31 16:14:06',
            'psp_NumPayments': '1',
            'psp_Currency': '032',
            'psp_TxSource': 'WEB',
            'psp_Product': '14',
            'psp_Country': 'ARG',
            'psp_Version': '2.2',
            'psp_MerchantId': merchant_id,
            'psp_Amount': "1000"
        }

        p2p_response = sdk.pay_online_2p(params)


        self.assertEqual("2", p2p_response.psp_ResponseCod)


    def test_response_of_p2p_with_billing_details(self):
        from nps_sdk.constants import DEVELOPMENT_ENV, SANDBOX_ENV
        import nps_sdk
        import logging
        import uuid
        merch_ref = uuid.uuid4()

        nps_sdk.Configuration.configure(environment=SANDBOX_ENV,
                                        secret_key="IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6",
                                        log_level=logging.INFO, debug=True, cert_verify_peer=False)

        sdk = nps_sdk.Nps()

        merchant_id = 'psp_test'

        params = {
                    "psp_Version": '2.2',
                    "psp_MerchantId": merchant_id,
                    "psp_TxSource": 'WEB',
                    "psp_MerchTxRef": merch_ref,
                    "psp_MerchOrderId": merch_ref,
                    "psp_Amount": '15050',
                    "psp_NumPayments": '1',
                    "psp_Currency": '032',
                    "psp_Country": 'ARG',
                    "psp_Product": '14',
                    "psp_CardNumber": '4850110000000000',
                    "psp_CardExpDate": '1712',
                    "psp_PosDateTime": '2016-12-01 12:00:00',
                    "psp_CardSecurityCode": '123',
                    "psp_CardHolderName": "Gustavo Diaz",
                    "psp_PurchaseDescription" : "Juguetes",
                    "psp_BillingDetails": {
                        "Invoice": "123",
                        "Invoice": "2017-01-01",
                        "InvoiceAmount": "123",
                        "InvoiceCurrency": "032",
                        "Person": {
                            'FirstName': 'John',
                            'LastName': 'Doe',
                            'MiddleName': 'Michael',
                            'PhoneNumber1': '+1 011 11111111',
                            'PhoneNumber2': '+1 011 22222222',
                            'DateOfBirth': '1979-01-12',
                            'Gender': 'M',
                            'Nationality': 'ARG',
                            'IDNumber': '54111111',
                            'IDType': '200'
                        },
                        'Address': {
                            'Street': 'Av. Collins',
                            'HouseNumber': '1245',
                            'AdditionalInfo': '2 A',
                            'StateProvince': 'Florida',
                            'City': 'Miami',
                            'Country': 'USA',
                            'ZipCode': '33140'
                        }
                    }
                  }

        resp = sdk.pay_online_2p(params)

        params = {
            "psp_Version": '2.2',
            "psp_MerchantId": merchant_id,
            "psp_QueryCriteria": 'T',
            "psp_QueryCriteriaId": resp.psp_TransactionId,
            "psp_PosDateTime": '2016-12-01 12:00:00'
        }

        resp = sdk.simple_query_tx(params)
        print resp


    def test_add_two_payment_methods_to_client(self):
        from nps_sdk.constants import DEVELOPMENT_ENV, SANDBOX_ENV
        import nps_sdk
        import logging
        import uuid
        merch_ref = uuid.uuid4()

        nps_sdk.Configuration.configure(environment=SANDBOX_ENV,
                                        #secret_key="IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6",
                                        secret_key="swGYxNeehNO8fS1zgwvCICevqjHbXcwPWAvTVZ5CuULZwKWaGPmXbPSP8i1fKv2q",
                                        log_level=logging.INFO, debug=True, cert_verify_peer=False)
        sdk = nps_sdk.Nps()
        merchant_id = 'sdk_test'

        params = {
            'psp_Version': '2.2',
            'psp_MerchantId': merchant_id,
            'psp_PosDateTime': '2017-01-01 12:00:00'
        }
        resp = sdk.create_client_session(params)

        params = {'psp_Address': {'AdditionalInfo': '2 A',
                                  'City': 'Miami',
                                  'Country': 'USA',
                                  'HouseNumber': '1245',
                                  'StateProvince': 'Florida',
                                  'Street': 'Av. Collins',
                                  'ZipCode': '33140'},
                  'psp_CardInputDetails': {'ExpirationDate': '1909',
                                           'HolderName': 'sol',
                                           'Number': '4242424242424243',
                                           'SecurityCode': '123'},
                  'psp_ClientSession': resp.psp_ClientSession,
                  'psp_MerchantId': merchant_id,
                  'psp_Person': {'DateOfBirth': '1979-01-12',
                                 'FirstName': 'John',
                                 'Gender': 'M',
                                 'IDNumber': '54111111',
                                 'IDType': '200',
                                 'LastName': 'Doe',
                                 'MiddleName': 'Michael',
                                 'Nationality': 'ARG',
                                 'PhoneNumber1': '+1 011 11111111',
                                 'PhoneNumber2': '+1 011 22222222'},
                  'psp_Product': '14',
                  'psp_Version': '2.2'}

        resp_cppt = sdk.create_payment_method_token(params)

        params = {
            'psp_Version': '2.2',
            'psp_MerchantId': merchant_id,
            'psp_EmailAddress': 'jhon.doe@example.com',
            'psp_AlternativeEmailAddress': 'jdoe@example.com',
            'psp_AccountID': 'jdoe78',
            'psp_AccountCreatedAt': '2010-10-23',
            'psp_PosDateTime': '2008-01-12 13:05:00',
            'psp_Person': {
                'FirstName': 'John',
                'LastName': 'Doe',
                'MiddleName': 'Michael',
                'PhoneNumber1': '+1 011 11111111',
                'PhoneNumber2': '+1 011 22222222',
                'DateOfBirth': '1979-01-12',
                'Gender': 'M',
                'Nationality': 'ARG',
                'IDNumber': '54111111',
                'IDType': '200'
            },
            'psp_Address': {
                'Street': 'Av. Collins',
                'HouseNumber': '1245',
                'AdditionalInfo': '2 A',
                'StateProvince': 'Florida',
                'City': 'Miami',
                'Country': 'USA',
                'ZipCode': '33140'
            }
        }

        resp_cc = sdk.create_customer(params)

        params = {
                    'psp_MerchantId': merchant_id,
                    'psp_CustomerId': resp_cc.psp_CustomerId,
                    'psp_PaymentMethod': {
                        'PaymentMethodToken': resp_cppt.psp_PaymentMethodToken
                    },
                    'psp_PosDateTime': '2008-01-12 13:05:00',
                    'psp_SetAsCustomerDefault': '1',
                    'psp_Version': '2.2'
                }

        resp_cpm1 = sdk.create_payment_method(params)
        resp_cpm1 = sdk.create_payment_method(params)

        params = {
            'psp_CustomerId': resp_cc.psp_CustomerId,
            'psp_MerchantId': merchant_id,
            'psp_PosDateTime': '2008-01-12 13:05:00',
            'psp_Version': '2.2'
        }

        resp = sdk.retrieve_customer(params)


    def test_get_customer(test):
        from nps_sdk.constants import DEVELOPMENT_ENV, SANDBOX_ENV
        import nps_sdk
        import logging
        import uuid
        merch_ref = uuid.uuid4()

        nps_sdk.Configuration.configure(environment=SANDBOX_ENV,
                                        secret_key="IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6",
                                        #secret_key="swGYxNeehNO8fS1zgwvCICevqjHbXcwPWAvTVZ5CuULZwKWaGPmXbPSP8i1fKv2q",
                                        log_level=logging.INFO, debug=True, cert_verify_peer=False)
        sdk = nps_sdk.Nps()
        merchant_id = 'psp_test'

        params = {
            'psp_CustomerId': '5hnNzGs2lMPw2ch33Yi0Ep334dDIoStQ',
            'psp_MerchantId': merchant_id,
            'psp_PosDateTime': '2008-01-12 13:05:00',
            'psp_Version': '2.2'
        }

        resp = sdk.retrieve_customer(params)


    def test_get_customer_with_proxy(test):
        from nps_sdk.constants import DEVELOPMENT_ENV, SANDBOX_ENV
        import nps_sdk
        import logging
        import uuid
        merch_ref = uuid.uuid4()
        nps_sdk.Configuration.configure(environment=SANDBOX_ENV,
                                        secret_key="IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6",
                                        #secret_key="swGYxNeehNO8fS1zgwvCICevqjHbXcwPWAvTVZ5CuULZwKWaGPmXbPSP8i1fKv2q",
                                        log_level=logging.INFO, debug=True, cert_verify_peer=False, proxy_url="http://10.129.10.70", proxy_port=3128)
        sdk = nps_sdk.Nps()
        merchant_id = 'psp_test'

        params = {
            'psp_CustomerId': '5hnNzGs2lMPw2ch33Yi0Ep334dDIoStQ',
            'psp_MerchantId': merchant_id,
            'psp_PosDateTime': '2008-01-12 13:05:00',
            'psp_Version': '2.2'
        }

        resp = sdk.retrieve_customer(params)

    def test_fuck(self):
        import requests
        s = requests.Session()
        s.headers = {'user-agent': 'my-test-script'}
        s.proxies = {'http': 'http://10.129.10.70:3128'}
        print s.get('http://mentos.com')



# -*- coding: utf-8 -*-
from nps_sdk.soap_client import SoapClient
from nps_sdk import services


class Nps(SoapClient):

    def pay_online_2p(self, params):
        resp = self._soap_call(services.PAY_ONLINE_2P, params)
        return resp

    def authorize_2p(self, params):
        resp = self._soap_call(services.AUTHORIZE_2P, params)
        return resp

    def query_txs(self, params):
        resp = self._soap_call(services.QUERY_TXS, params)
        return resp

    def simple_query_tx(self, params):
        resp = self._soap_call(services.SIMPLE_QUERY_TX, params)
        return resp

    def refund(self, params):
        resp = self._soap_call(services.REFUND, params)
        return resp

    def capture(self, params):
        resp = self._soap_call(services.CAPTURE, params)
        return resp

    def authorize_3p(self, params):
        resp = self._soap_call(services.AUTHORIZE_3P, params)
        return resp

    def bank_payment_3p(self, params):
        resp = self._soap_call(services.BANK_PAYMENT_3P, params)
        return resp

    def bank_payment_2p(self, params):
        resp = self._soap_call(services.BANK_PAYMENT_2P, params)
        return resp

    def cash_payment_3p(self, params):
        resp = self._soap_call(services.CASH_PAYMENT_3P, params)
        return resp

    def change_secret_key(self, params):
        resp = self._soap_call(services.CHANGE_SECRET_KEY, params)
        return resp

    def fraud_screening(self, params):
        resp = self._soap_call(services.FRAUD_SCREENING, params)
        return resp

    def notify_fraud_screening_review(self, params):
        resp = self._soap_call(services.NOTIFY_FRAUD_SCREENING_REVIEW, params)
        return resp

    def pay_online_3p(self, params):
        resp = self._soap_call(services.PAY_ONLINE_3P, params)
        return resp

    def split_authorize_3p(self, params):
        resp = self._soap_call(services.SPLIT_AUTHORIZE_3P, params)
        return resp

    def split_pay_online_3p(self, params):
        resp = self._soap_call(services.SPLIT_PAY_ONLINE_3P, params)
        return resp

    def query_card_number(self, params):
        resp = self._soap_call(services.QUERY_CARD_NUMBER, params)
        return resp

    def get_iin_details(self, params):
        resp = self._soap_call(services.GET_IIN_DETAILS, params)
        return resp

    def create_payment_method(self, params):
        resp = self._soap_call(services.CREATE_PAYMENT_METHOD, params)
        return resp

    def create_payment_method_from_payment(self, params):
        resp = self._soap_call(services.CREATE_PAYMENT_METHOD_FROM_PAYMENT, params)
        return resp

    def retrieve_payment_method(self, params):
        resp = self._soap_call(services.RETRIEVE_PAYMENT_METHOD, params)
        return resp

    def update_payment_method(self, params):
        resp = self._soap_call(services.UPDATE_PAYMENT_METHOD, params)
        return resp

    def delete_payment_method(self, params):
        resp = self._soap_call(services.DELETE_PAYMENT_METHOD, params)
        return resp

    def create_customer(self, params):
        resp = self._soap_call(services.CREATE_CUSTOMER, params)
        return resp

    def retrieve_customer(self, params):
        resp = self._soap_call(services.RETRIEVE_CUSTOMER, params)
        return resp

    def update_customer(self, params):
        resp = self._soap_call(services.UPDATE_CUSTOMER, params)
        return resp

    def delete_customer(self, params):
        resp = self._soap_call(services.DELETE_CUSTOMER, params)
        return resp

    def recache_payment_method_token(self, params):
        resp = self._soap_call(services.RECACHE_PAYMENT_METHOD_TOKEN, params)
        return resp

    def create_payment_method_token(self, params):
        resp = self._soap_call(services.CREATE_PAYMENT_METHOD_TOKEN, params)
        return resp

    def retrieve_payment_method_token(self, params):
        resp = self._soap_call(services.RETRIEVE_PAYMENT_METHOD_TOKEN, params)
        return resp

    def create_client_session(self, params):
        resp = self._soap_call(services.CREATE_CLIENT_SESSION, params)
        return resp

    def get_installments_options(self, params):
        resp = self._soap_call(services.GET_INSTALLMENTS_OPTIONS, params)
        return resp

    def split_pay_online_2p(self, params):
        resp = self._soap_call(services.SPLIT_PAY_ONLINE_2P, params)
        return resp

    def split_authorize_2p(self, params):
        resp = self._soap_call(services.SPLIT_AUTHORIZE_2P, params)
        return resp

    def query_card_details(self, params):
        resp = self._soap_call(services.QUERY_CARD_DETAILS, params)
        return resp


key_config = {
    'psp_Person.FirstName.max_length':128,
    'psp_Person.LastName.max_length':64,
    'psp_Person.MiddleName.max_length':64,
    'psp_Person.PhoneNumber1.max_length':32,
    'psp_Person.PhoneNumber2.max_length':32,
    'psp_Person.Gender.max_length':1,
    'psp_Person.Nationality.max_length':3,
    'psp_Person.IDNumber.max_length':40,
    'psp_Person.IDType.max_length':5,
    'psp_Address.Street.max_length':128,
    'psp_Address.HouseNumber.max_length':32,
    'psp_Address.AdditionalInfo.max_length':128,
    'psp_Address.City.max_length':40,
    'psp_Address.StateProvince.max_length':40,
    'psp_Address.Country.max_length':3,
    'psp_Address.ZipCode.max_length':10,
    'psp_OrderItem.Description.max_length':127,
    'psp_OrderItem.Type.max_length':20,
    'psp_OrderItem.SkuCode.max_length':48,
    'psp_OrderItem.ManufacturerPartNumber.max_length':30,
    'psp_OrderItem.Risk.max_length':1,
    'psp_Leg.DepartureAirport.max_length':3,
    'psp_Leg.ArrivalAirport.max_length':3,
    'psp_Leg.CarrierCode.max_length':2,
    'psp_Leg.FlightNumber.max_length':5,
    'psp_Leg.FareBasisCode.max_length':15,
    'psp_Leg.FareClassCode.max_length':3,
    'psp_Leg.BaseFareCurrency.max_length':3,
    'psp_Passenger.FirstName.max_length':50,
    'psp_Passenger.LastName.max_length':30,
    'psp_Passenger.MiddleName.max_length':30,
    'psp_Passenger.Type.max_length':1,
    'psp_Passenger.Nationality.max_length':3,
    'psp_Passenger.IDNumber.max_length':40,
    'psp_Passenger.IDType.max_length':10,
    'psp_Passenger.IDCountry.max_length':3,
    'psp_Passenger.LoyaltyNumber.max_length':20,
    'psp_SellerDetails.IDNumber.max_length':40,
    'psp_SellerDetails.IDType.max_length':10,
    'psp_SellerDetails.Name.max_length':128,
    'psp_SellerDetails.Invoice.max_length':32,
    'psp_SellerDetails.PurchaseDescription.max_length':32,
    'psp_SellerDetails.MCC.max_length':5,
    'psp_SellerDetails.ChannelCode.max_length':3,
    'psp_SellerDetails.GeoCode.max_length':5,
    'psp_TaxesRequest.TypeId.max_length':5,
    'psp_MerchantAdditionalDetails.Type.max_length':1,
    'psp_MerchantAdditionalDetails.SdkInfo.max_length':48,
    'psp_MerchantAdditionalDetails.ShoppingCartInfo.max_length':48,
    'psp_MerchantAdditionalDetails.ShoppingCartPluginInfo.max_length':48,
    'psp_CustomerAdditionalDetails.IPAddress.max_length':45,
    'psp_CustomerAdditionalDetails.AccountID.max_length':128,
    'psp_CustomerAdditionalDetails.DeviceFingerPrint.max_length':4000,
    'psp_CustomerAdditionalDetails.BrowserLanguage.max_length':2,
    'psp_CustomerAdditionalDetails.HttpUserAgent.max_length':255,
    'psp_BillingDetails.Invoice.max_length':32,
    'psp_BillingDetails.InvoiceCurrency.max_length':3,
    'psp_ShippingDetails.TrackingNumber.max_length':24,
    'psp_ShippingDetails.Method.max_length':3,
    'psp_ShippingDetails.Carrier.max_length':3,
    'psp_ShippingDetails.GiftMessage.max_length':200,
    'psp_AirlineDetails.TicketNumber.max_length':14,
    'psp_AirlineDetails.PNR.max_length':10,
    'psp_VaultReference.PaymentMethodToken.max_length':64,
    'psp_VaultReference.PaymentMethodId.max_length':64,
    'psp_VaultReference.CustomerId.max_length':64
    }


import os
import sys

sdk = {'language': 'Python', 'version': '1.0'}
sys_ver = sys.version[0:7]
api = {'version': '2.2'}
PRODUCTION_ENV = 0
STAGING_ENV = 1
SANDBOX_ENV = 2
DEVELOPMENT_ENV = 3
wsdl_folder = "/wsdl/"
dir = os.path.dirname(__file__) + wsdl_folder
_FILE_SCHEMA = "file://"
_PRODUCTION_WSDL_NAME = "production.wsdl"
_SANDBOX_WSDL_NAME = "sandbox.wsdl"
_STAGING_WSDL_NAME = "staging.wsdl"
_DEVELOPMENT_WSDL_NAME = "development.wsdl"

_PRODUCTION_URL = _FILE_SCHEMA + os.path.join(dir, _PRODUCTION_WSDL_NAME)
_SANDBOX_URL = _FILE_SCHEMA + os.path.join(dir, _SANDBOX_WSDL_NAME)
_STAGING_URL = _FILE_SCHEMA + os.path.join(dir, _STAGING_WSDL_NAME)
_DEVELOPMENT_URL = _FILE_SCHEMA + os.path.join(dir, _DEVELOPMENT_WSDL_NAME)
DEF_TIMEOUT = 60


from nps_sdk.constants import PRODUCTION_ENV, STAGING_ENV, SANDBOX_ENV, DEVELOPMENT_ENV
from nps_sdk.errors import ApiException
import nps_sdk
import logging


merchant_id = "psp_test"
#merchant_id = "sdk_test"
#merchant_id = "365online"

nps_sdk.Configuration.configure(environment=SANDBOX_ENV,
                                # secret_key="swGYxNeehNO8fS1zgwvCICevqjHbXcwPWAvTVZ5CuULZwKWaGPmXbPSP8i1fKv2q", #sdk_test
                                secret_key="IeShlZMDk8mp8VA6vy41mLnVggnj1yqHcJyNqIYaRINZnXdiTfhF0Ule9WNAUCR6", #psp_test
                                #secret_key="xfMzDG2gDMwYABT3JWUmkH14i3uwXMTW2hI2GIomhxq3FORdhDs9EDSAeqRQJmMt", #365online
                                #secret_key="wDbxnRDvF3wmcETN9bih7j4R9FhC3PczaBuNd7JqAFalR38adqGiYKAfZpsbCSYm", #inclufin
                                #secret_key="0NdyJ37jbRqUnw5ATJoDbkq52WYv9BY8YC6qDCZodNIOJLYmcL0D5oG5Kp0R0WZZ",
                                log_level=logging.INFO, debug=True, cert_verify_peer=False, cert=False, timeout=30, cache=True, cache_location='/tmp', cache_ttl=86400, as_obj=True)


def run_create_client_session():
    """CreateClientSession"""

    sdk = nps_sdk.Nps()

    params = {
        'psp_Version': '2.2',
        'psp_MerchantId': merchant_id,
        'psp_PosDateTime': '2017-01-01 12:00:00'
    }
    resp = sdk.create_client_session(params)

    return resp



print(run_create_client_session())


from nps_sdk.configuration import Configuration
from nps_sdk.sdk import Nps
import os
os.environ['REQUESTS_CERT_NPS'] = os.path.dirname(__file__) + "/certs/sample.pem"
os.environ['REQUESTS_CERT_KEY'] = os.path.join(os.path.dirname(__file__)  + '/certs/','sample.key')


import collections
import hashlib
import requests
import suds.transport as transport
from nps_sdk import services
from nps_sdk.constants import sdk
from nps_sdk.conf import sanititize_struc
from suds.plugin import MessagePlugin
import logging
import re
from nps_sdk.version import VERSION

_log_format = '%(asctime)s - NpsSDK - %(levelname)s - %(message)s'

class Enum(set):
    def __getattr__(self, name):
        if name in self:
            return name
        raise AttributeError

try:
    from StringIO import StringIO as ios
except ImportError:
    from io import BytesIO as ios

__all__ = ['RequestsTransport']


class RequestsTransport(transport.Transport):
    def __init__(self, session=None, **kwargs):
        transport.Transport.__init__(self)
        self._session = session or requests.Session()
        self._kwargs = kwargs

    def open(self, request):
        resp = self._session.get(request.url, timeout=self._kwargs.get('timeout', 60))
        resp.raise_for_status()
        return ios(resp.content)

    def send(self, request):
        resp = self._session.post(
            request.url,
            data=request.message,
            headers=request.headers,
            json=None, timeout=self._kwargs.get('timeout', 60))

        if resp.headers.get('content-type') not in ('text/xml',
                                                    'application/soap+xml'):
            resp.raise_for_status()

        return transport.Reply(
            resp.status_code,
            resp.headers,
            resp.content,
        )

def add_extra_info(service, params):
    if service in services.get_merch_det_not_add_services():
        return params
    info = {"SdkInfo": sdk.get('language') + ' SDK Version: ' + VERSION}

    if "psp_MerchantAdditionalDetails" in params.keys():
        params.get("psp_MerchantAdditionalDetails").update(info)
    else:
        params.update({"psp_MerchantAdditionalDetails": info})
    return params

def add_secure_hash(params, secret_key):
    secure_hash = _create_secure_hash(params, secret_key)
    params.update({"psp_SecureHash": secure_hash})
    return params


def _create_secure_hash(params, secret_key):
    m = hashlib.md5()
    od = collections.OrderedDict(sorted(params.items()))
    concatenated_data = "".join([str(x).strip() for x in od.values() if type(x) is not dict and type(x) is not list]) + secret_key
    concatenated_data = concatenated_data.encode('utf-8')
    m.update(concatenated_data)
    coded = m.hexdigest()
    return coded


def _check_sanitize(params, is_root=False, nodo = None):
    if is_root:
        result_params = {}
    else:
        result_params = params
    for k, v in params.items():
        if type(v) is dict:
            result_params.update({k: _check_sanitize(v, nodo=k)})
        elif type(v) is list:
            result_params.update({k: check_sanitize_array(v, nodo=k)})
        else:
            result_params.update({k: _validate_size(v, k, nodo)})

    return result_params


def check_sanitize_array(params, nodo):
    result_params = []
    for x in params:
        result_params.append(_check_sanitize(x, nodo=nodo))
    return result_params


def _validate_size(value, k=None, nodo=None):
    if nodo is not None:
        key_name = nodo + "." + k + ".max_length"
    else:
        key_name = k + ".max_length"
    size = sanititize_struc.key_config.get(key_name)
    return str(value)[0:size]


def _mask_data(data):
    data = data.decode('utf-8')
    data = _mask_c_number(data)
    data = _mask_exp_date(data)
    data = _mask_cvc(data)
    data = _mask_tokenization_c_number(data)
    data = _mask_tokenization_exp_date(data)
    data = _mask_tokenization_cvc(data)

    return data


def _mask_c_number(data):
    c_number_key = "</psp_CardNumber>"
    c_numbers = _find_c_numbers(data, c_number_key)
    for c_number in c_numbers:
        c_number_len = len(c_number[0: len(c_number) - len(c_number_key)])
        masked_chars = c_number_len - 10
        data = data.replace(c_number,
                            c_number[0: 6] + "*"*masked_chars + c_number[len(c_number)-4-len(c_number_key): len(c_number)])

    return data

def _mask_exp_date(data):
    exp_date_key = "</psp_CardExpDate>"
    exp_dates = _find_exp_date(data, exp_date_key)
    for exp_date in exp_dates:
        data = data.replace(exp_date, "****" + exp_date_key)
    return data

def _mask_cvc(data):
    cvc_key = "</psp_CardSecurityCode>"
    cvcs = _find_cvc(data, cvc_key)
    for cvc in cvcs:
        cvc_len = len(cvc[0: len(cvc) - len(cvc_key)])
        data = data.replace(cvc, "*"*cvc_len + cvc_key)
    return data

def _mask_tokenization_c_number(data):
    c_number_key = "</Number>"
    c_numbers = _find_c_numbers(data, c_number_key)
    for c_number in c_numbers:
        c_number_len = len(c_number[0: len(c_number) - len(c_number_key)])
        masked_chars = c_number_len - 10
        data = data.replace(c_number,
                            c_number[0: 6] + "*"*masked_chars + c_number[len(c_number)-4-len(c_number_key): len(c_number)])

    return data

def _mask_tokenization_exp_date(data):
    exp_date_key = "</ExpirationDate>"
    exp_dates = _find_exp_date(data, exp_date_key)
    for exp_date in exp_dates:
        data = data.replace(exp_date, "****" + exp_date_key)
    return data

def _mask_tokenization_cvc(data):
    cvc_key = "</SecurityCode>"
    cvcs = _find_cvc(data, cvc_key)
    for cvc in cvcs:
        cvc_len = len(cvc[0: len(cvc) - len(cvc_key)])
        data = data.replace(cvc, "*"*cvc_len + cvc_key)
    return data


def _find_c_numbers(data, key):
    c_numbers = re.findall("\d{13,19}" + key, data)
    return c_numbers


def _find_exp_date(data, key):
    exp_dates = re.findall("\d{4}" + key, data)
    return exp_dates


def _find_cvc(data, key):
    cvcs = re.findall("\d{3,4}" + key, data)
    return cvcs


def _parse_to_xml(text):
    import xml.dom.minidom
    xml = xml.dom.minidom.parseString(text)
    pretty_xml_as_string = xml.toprettyxml()
    return pretty_xml_as_string

def get_log_format():
    return _log_format


def get_builded_proxy_url(url, port):
    return {"https": "".join([url, ":", str(port)])}


class LogPlugin(MessagePlugin):

    def sending(self, context):
        logging.info(_parse_to_xml(_mask_data(context.envelope)))

    def received(self, context):
        logging.info(_parse_to_xml(_mask_data(context.reply)))

class LogPlugin(MessagePlugin):

    def sending(self, context):
        logging.info(_parse_to_xml(context.envelope))

    def received(self, context):
        logging.info(_parse_to_xml(context.reply))


class OutgoingFilter(logging.Filter):
    def filter(self, record):
        return record.msg.startswith('sending:')

def recursive_asdict(d):
    """Convert Suds object into serializable format."""
    from suds.sudsobject import asdict
    out = {}
    # out = OrderedDict()
    for k, v in asdict(d).items():
        if hasattr(v, '__keylist__'):
            out[k] = recursive_asdict(v)
        elif isinstance(v, list):
            out[k] = []
            for item in v:
                if hasattr(item, '__keylist__'):
                    out[k].append(recursive_asdict(item))
                elif not isinstance(item, list):
                    out[k] = item
                else:
                    out[k].append(str(item))
        else:
            out[k] = str(v)
    return out